<?php

namespace App\Http\Controllers;

use App\Models\Size;
use App\Models\Order;
use App\Models\Product;
use Illuminate\View\View;
use App\Mail\OrderCompleted;
use App\Models\DeliveryInfo;
use Illuminate\Http\Request;
use Illuminate\View\Factory;
use App\Traits\PaystackRequests;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;
use Illuminate\Support\Facades\Mail;
use App\PaymentGatewayServices\Paystack;
use Gloudemans\Shoppingcart\Facades\Cart;
use App\Exceptions\InvalidFinancialSettlementFromPGS;

class PaystackController extends Controller
{
    use PaystackRequests;
    //

    /**
     * testing purposes
     * we use this method to initiate a payment with paystack api anfd we get a payment url 
     * and reference from paystack
     */
    public function initiateTest()
    {
        $paystackInitUrl = 'https://api.paystack.co/transaction/initialize';
        try {
            $response = Http::withHeaders($this->getPaystackAuthHeaders())
                ->post($paystackInitUrl, [
                    'email' => 'meetdaniels@gmail.com',
                    'amount' => '2000'
                ]);
        } catch (\Throwable $th) {
            //    return redirect('/');
            dd('paystack could not connect');
        }
        // $response = Http::withHeaders($this->getPaystackAuthHeaders())
        //     ->post($paystackInitUrl, [
        //         'email' => 'meetdaniels@gmail.com',
        //         'amount' => '2000'
        //     ]);
        if (!$response) {
            // dd('femi');
        }
        // dd($response, $response->body());
    }

    /**
     * testing purposes
     * we use this method to verify a payment with paystack api, we get the trx status and metadata for 
     * this payment transaction. we also get other paymentRelatedData that we might needand reference from paystack
     */
    public function verifyTest()
    {
        $reference = 'create2fit_6661dc6e9dc94';
        $paystackVerifyUrl = Paystack::VERIFY_URL_BASE . $reference;

        $response = Http::withHeaders($this->getPaystackAuthHeaders())
            ->get($paystackVerifyUrl);

        dd($response, $response->body());

    }


    public function callback(Request $request)
    {
        //we only need the reference from the callback request after payment has been made
        /**
         * @var string $pgs_ref refers to reference from the callback request after 
         * payment has been made.
         * which is all we need to verify the transaction
         */
        $pgs_ref = $request->reference;

        // next we validate that reference, cos we don't know if it's successful or failed
        /**
         * @var array $pgsData contains dataFromPaymentVerification. it contains the
         *  ['orderId','amountPaid', 'fees', 'status'] 
         */
        $pgsData = Paystack::verifyPayment($pgs_ref);

        $relatedOrder = Order::findOrFail($pgsData['orderId']);

        if ($relatedOrder->status !== 'success' || $relatedOrder->tx_ref !== $pgs_ref) {
            $validFinancialSettlement = Paystack::validateFinancialSettlement($pgsData, $relatedOrder);

            $this->handleFinancialSettlementIssues($validFinancialSettlement, $pgsData, $pgs_ref);

            $relatedOrder = $this->markPendingOrderAsCompleted($relatedOrder, $pgsData, $pgs_ref);

            // this doesn't work $relatedOrder->deliveryInfo, but this works $relatedOrder->deliveryInfo()->first();
            if (blank($relatedOrder->deliveryInfo()?->first()?->id)) {
                $relatedOrder->deliveryInfo()->create(session('delivery_info'));
            }
            $this->updateInventoryAfterPurchase($relatedOrder);
            $this->emailClientWithOrderInfo($relatedOrder);
        }
        return $this->providePositiveResponse($relatedOrder, $pgsData);
    }


    // Helpers
    private function handleFinancialSettlementIssues($validFinancialSettlement, $pgsData, $pgs_ref)
    {
        if (!$validFinancialSettlement) {
            $errorMsg = "Transaction with reference {$pgs_ref} was : {$pgsData['status']} . ";
            $errorMsg .= "However there were inconsistencies found in payment fees or amount @paystack";
            report(new InvalidFinancialSettlementFromPGS($errorMsg, $pgsData));
        }
    }

    private function markPendingOrderAsCompleted($relatedOrder, $pgsData, $pgs_ref)
    {
        $relatedOrder->update([
            'status' => $pgsData['status'], //should resolve to success
            'tx_ref' => $pgs_ref,
            'pgs_tx_id' => $pgs_ref,
        ]);
        return $relatedOrder;
    }


    public function emailClientWithOrderInfo($relatedOrder)
    {
        // send mail to the client about successful transaction
        Mail::to($relatedOrder->user->email)
            ->bcc([config('app.dev_email'),])
            ->send(new OrderCompleted($relatedOrder));
    }


    public function updateInventoryAfterPurchase(Order $order)
    {
        foreach (json_decode($order->content,1) as $rowId => $orderItem) {
            // dd($orderItem['options']['size'], $orderItem, $rowId);
            $sizeName = Size::whereName($orderItem['options']['size'] ?? 'm')->first()->name;
            $sizeId = Size::whereName($sizeName)->first()->id;
            $relatedProduct = Product::find($orderItem['id']);

            $prevProductQty = $relatedProduct->sizes()->where('size_id', $sizeId)->first()->pivot->qty;
            // dd($sizeId, $prevProductQty, $relatedProduct->sizes()->where('size_id', $sizeId)->first());
            $newQty = (int) $prevProductQty - (int) $orderItem['qty'];

            // update the individual sizeId
            $relatedProduct->sizes()->updateExistingPivot($sizeId, [
                'qty' => $newQty,
            ]);
            // update the general inventory count
            (new ProductController())->updateInventory($relatedProduct);
        }
    }


    /**
     * provides a positive response to the user
     * @param mixed $relatedOrder
     * 
     * @return Factory
     */
    private function providePositiveResponse($relatedOrder, $pgsData): Factory|View
    {
        #####################################################################################
        ##### ALL IS WELL (NOW WE DESTROY CART AND REDIRECT BACK WITH SUCCESS MESSAGE) ######
        #####################################################################################
        Cart::destroy();

        flash('Payment Successful');

        return view('payment.successful', compact('relatedOrder', 'pgsData')); // generate receipts (using provided view)
    }

    public function canceledByUser(Request $request)
    {
        return view('payment.successful');
    }
}
