<?php

namespace App\Http\Controllers;

use App\Models\Size;
use App\Models\Order;
use App\Models\Product;
use Illuminate\View\View;
use Illuminate\Http\Request;
use Illuminate\View\Factory;
use App\Models\ProductCollection;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\RedirectResponse;
use App\Http\Requests\StoreProductRequest;
use App\Traits\Controllers\ProductControllerHelpers;

class ProductController extends Controller
{
    use ProductControllerHelpers;
    public function __construct()
    {
        $this->middleware('auth')->except(['index', 'show']);
    }


    /**
     * Display a listing of the resource.
     */
    public function index(Request $request): Factory|View
    {
        $queryBuilder = Product::query();
        // $queryBuilder = Product::where('name', 'like', "%{$searchfor}%");
        $allProducts = match ($request->sort) {
            'a-z' => $queryBuilder->orderBy('name')->get(),
            'z-a' => $queryBuilder->orderByDesc('name')->get(),
            'lowest' => $queryBuilder->orderBy('price')->get(),
            'highest' => $queryBuilder->orderByDesc('price')->get(),
            'oldest' => $queryBuilder->orderBy('created_at')->get(),
            'latest' => $queryBuilder->orderByDesc('created_at')->get(),
            default => $queryBuilder->orderBy('name')->get(),
        };
        return view('website.products.index', compact('allProducts'));
    }

    public function adminIndex(): Factory|View
    {
        $products = Product::all();

        // dd($products);
        return view('admin.products.index', compact('products'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create(): Factory|View
    {
        $sizes = Size::all();
        $allCategories = ProductCollection::all()->sortBy('name');
        return view('admin.products.create', compact('allCategories', 'sizes'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(StoreProductRequest $request): RedirectResponse
    {
        // dd(now()->toDateTimeLocalString());
        $newProduct = $this->storeNewProduct($request);
        $this->handleProductSizes($request, $newProduct);
        $this->handleProductImages($request, $newProduct);
        $createdResponse = $this->providePositiveResponse($newProduct);

        // final hosekeeping
        $this->finalHousekeeping($newProduct);

        return $createdResponse;
    }


    /**
     * Display the specified resource.
     */
    public function show(Product $product): Factory|View
    {
        return view('website.products.show', compact('product'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(Product $product)
    {
        // $sizes = $product->sizes;
        $sizes = Size::all();
        $allCategories = ProductCollection::all()->sortBy('name');
        return view('admin.products.edit', compact('allCategories', 'sizes', 'product'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(StoreProductRequest $request, Product $product)
    {
        $this->updateProductModel($request, $product);

        $updateResponse = $this->updateExistingProductRelatedData($request, $product);
        
        $this->handleProductImages($request, $product);

        $this->finalHousekeeping($product);

        return $updateResponse;
    }



    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Product $product)
    {
        $product->delete()
            ? flash('Product Deleted 👍')
            : flash()->addError('Oops! Error occured, Try Again');

        return back();
    }



    public function updateProductModel($request, $product)
    {
        $validated = collect($request->validated());
        $safeData = $validated->only(['name', 'price', 'description'])->toArray();

        $product->update($safeData);
    }
    public function finalHousekeeping($product)
    {
        $product->updateSlug();
        $this->updateInventory($product);
    }


    public function updateInventory(Product $product)
    {
        $sumOfSizesQty = 0;

        foreach (Size::all() as $size) {
            $sumOfSizesQty += $product?->getStockQtyForSize($size);
        }
        // dd('sumof', $sumOfSizesQty);
        $product->inventory = $sumOfSizesQty;
        $product->save();

    }
}
