<?php
namespace App\PaymentGatewayServices;

use App\Models\Order;
use Illuminate\View\View;
use App\Models\Pgs;
use App\Models\SubscriptionPlan;
use App\Traits\PaystackRequests;
use Illuminate\Support\Facades\Http;
use App\Contracts\Payment\PaymentGatewayServiceContract;

class Paystack implements PaymentGatewayServiceContract
{
    use PaystackRequests;

    /**
     * the url used by paystack to initiate a payment
     */
    const INIT_URL = 'https://api.paystack.co/transaction/initialize';

    /**
     * the url used by paystack to verify a payment. This url requires the reference of the trx
     * you want to verify to be appended to it. 
     * usage: Paystack::VERIFY_URL . $reference
     * where $reference is the trx ref of the order you want to track.
     */
    const VERIFY_URL_BASE = "https://api.paystack.co/transaction/verify/";

    /**
     * Refers to the transaction reference of any given payment or transaction made on paystack
     */
    public $reference;

    protected Order $order;

    public function __construct()
    {

    }

    public function receivePaymentFor(Order $order)
    {
        $this->order = $order;
        // since paystack is receiving payment for this order,
        //  it's normal to set the pgs_id to paystack's id
        $pgs_id = Pgs::whereName('paystack')->first()->id;

        $this->order->update(['pgs_id' => $pgs_id]); // set the pgs_id
    }


    /**
     * {REFACTOR} REQUIRED
     * we use this method to verify a payment with paystack api, we get the trx status and metadata for 
     * this payment transaction. we also get other paymentRelatedData that we might need 
     * and  we also get transaction reference from paystack which is usually the same between
     * them and us. (reference can be paystack generated or also generated by us)
     * 
     * This method returns an array of data from the verified transaction 
     * if it was successful and a string if it failed
     * 
     * NB: data returned from this method contains already formatted amount and fees, they've been converted 
     * from kobo to naira respectively and hence, further tests should use them as they are. 
     * @return array 
     */
    public static function verifyPayment(string $pgs_ref): array|View
    {
        self::validatePgsRefWasReceived($pgs_ref); //refactor exception handler

        $relatedOrder = Order::where('identifier', $pgs_ref)->first();
        // dd($relatedOrder, $pgs_ref);
        $data = self::verifyTrxWithPaystack($pgs_ref);

        if ($data['status'] == 'success') {
            // we can start gathering relevant data
            $orderId = $relatedOrder->id;
            $amountPaid = self::formatAmountDown($data['amount']);
            $fees = self::formatAmountDown($data['fees']);
            $status = $data['status'];

            self::storePaymentInfoForOrder($data, $relatedOrder);

            if ($orderId and $amountPaid and $fees and $status)
                return $pgsData = compact('orderId', 'amountPaid', 'fees', 'status');
        }

        // handle failed transactions : return view for failed transactions
        $relatedOrder->update([
            'status' => 'failed'
        ]);
        return view('payment.successful', compact('relatedOrder'));
    }


    /**
     * 
     * Returns payment data from paystack relating to $pgs_ref
     * 
     * @param string $pgs_ref refers to the transaction reference 
     * we want to verify with paystack
     * 
     * @return array
     */
    public static function verifyTrxWithPaystack($pgs_ref): array
    {
        $paystackVerifyUrl = Paystack::VERIFY_URL_BASE . $pgs_ref;

        $response = Http::withHeaders(self::getPaystackHeaders())->get($paystackVerifyUrl);
        $resBody = json_decode($response->body(), true);
        return $data = (!$resBody['status']) ? [] : $resBody['data'];
    }

    public static function validatePgsRefWasReceived($pgs_ref)
    {
        if (!$pgs_ref) {
            $exceptionMsg = "no transaction reference provided for verification:";
            $exceptionMsg .= "transaction reference is missing";

            throw new \Exception($exceptionMsg);
        }
    }
    /**
     * This methods returns all the data keys for payment card/bank info
     * eg. card number, ccv, card type, card bin etc.
     * @return [type]
     */
    public static function getAuthorizationDataKeysFromPgs()
    {
        $keyz = ['bin', 'last4', 'exp_month', 'exp_year', 'channel', 'card_type'];
        // array_push($keyz, 'receiver_bank', 'receiver_bank_account_number');
        return $keyz;
    }


    public static function storePaymentInfoForOrder($data, $relatedOrder)
    {
        // let's gather authorization data eg. bank, card info (payment medium)
        foreach (self::getAuthorizationDataKeysFromPgs() as $key)
            $paymentInfo[$key] = $$key = $data['authorization'][$key];

        // dd($relatedOrder);
        $relatedOrder->paymentInfo()->create($paymentInfo);
    }
    /**
     * this helps us to know if we're not being scammed
     * @param mixed $data refers to the data we got from the payment verification
     * @param mixed $relatedOrder refers to the order associated with this payment
     * 
     * @return bool
     */
    public static function validateFinancialSettlement(array $data, order $relatedOrder): bool
    {
        $checkAmountPaid = (string) $relatedOrder->getTotalAmountAsFloat() === (string) $data['amountPaid'];
        
        // peculiar to paystack : paystack uses 2dp
        $checkFees = (string) round(self::getFeesForOrder($relatedOrder), 2) === (string) $data['fees'];
        return $checkAmountPaid and $checkFees;
    }



    /**
     * Returns the fees for any payment gateway provided
     * 
     * @param mixed $order
     * 
     * @return float
     */
    public static function getFeesForOrder(order $order): float
    {
        $percentCharge = (1.5 / 100); //1.5%
        $fixedCharge = 100;

        $percentageCharged = $order->getTotalAmountAsFloat() * $percentCharge; //1.5%

        return $finalCharge = ($order->getTotalAmountAsFloat() > 2000)
            ? $percentageCharged + $fixedCharge
            : $percentageCharged;
    }

    /**
     * @return [type]
     */
    public function getDataForPaymentInitialization()
    {
        $reference = $this->generateTransactionReference();
        return [
            'email' => $this->order->user->email,
            'amount' => $this->formatAmountUp($this->order->getTotalAmountAsFloat()),
            'reference' => $reference,
            'callback_url' => route('paystack.callback'),
            // 'metadata' => $this->getMetadata($this->order->subscriptionPlan),
        ];
    }

    // private function getRedirectUrl()
    // {
    //     $scheme = 'https://';
    //     $host = '127.0.0.1:5173';
    //     $domain = $scheme . $host;
    //     $routePath = '/paystack/callback';
    //     $url = $domain . $routePath;

    //     if (config('app.env') == 'local') {
    //         return $url;
    //     }
    // }

    /**
     * This generates a reference for our transaction inHouse
     */
    private function generateTransactionReference(): string
    {
        return $this->order->identifier;
    }



    /**
     * @param SubscriptionPlan $relatedSubscriptionPlan
     * 
     * @return string
     */
    public function getMetadata(SubscriptionPlan $relatedSubscriptionPlan): string
    {
        $metaData = [
            'order_id' => $this->order->id,
            "cancel_action" => route('paystack.cancel_action'),

            'custom_fields' => [
                [
                    "display_name" => "Subscription Plan Title",
                    "variable_name" => "subscription_plan_title",
                    "value" => $relatedSubscriptionPlan->title,
                ],

                [
                    "display_name" => "Subscription Plan Validity",
                    "variable_name" => "subscription_plan_validity",
                    "value" => $relatedSubscriptionPlan->validity,
                ],

                [
                    "display_name" => "Subscription Activation Date",
                    "variable_name" => "subscription_activation_date",
                    "value" => $this->order->check_in_date,
                ],
            ]
        ];
        return json_encode($metaData);
    }

    /**
     * we use this method to initiate a payment with paystack api anfd we get a payment url 
     * and reference from paystack
     */
    public function initiatePayment(): bool|string
    {
        try {
            $response = Http::withHeaders(self::getPaystackHeaders())
                ->post(self::INIT_URL, $this->getDataForPaymentInitialization());
            if (!$response) {
                throw new \Exception("couldn't connect to paystack at {now()}");
            }
        } catch (\Throwable $th) {
            dd($th->getMessage());
        }

        $resBody = json_decode($response->body(), true);
        // dd($resBody);
        $data = $resBody['data'];

        $this->reference = $data['reference'];

        return $data['authorization_url'] ?: false;

    }

    /**
     * this method returns a redirect to paystack's paymentUrl
     */
    public function acceptPayment()
    {
        $paymentUrl = $this->initiatePayment();

        return redirect($paymentUrl);
    }



    public function getOrder(order $order)
    {

    }


    /**
     * this function Converts [naira to kobo, dollar to cents etc]
     * synopsis: 
     * since paystack uses the subunit of any currency  eg. kobo, cents instead of using the 
     *  main unit such as naira and dollars etc.
     * we have to convert from mainUnit(₦) to the subUnit(kobo) of currency transacted cos, we can't tell
     * our subscribers to put their value in kobo
     * hence, we multiply by 100. therefore, ₦3000 => 3000*100 (in kobo)
     */
    public function formatAmountUp(float|string $amount)
    {
        return (float) $amount * 100;
    }

    /**
     * this function Converts [kobo to naira, cents to dollars etc]
     * synopsis: 
     * since paystack uses the subunit of any currency  eg. kobo, cents instead of using the 
     *  main unit such as naira and dollars etc.
     * we have to convert from subUnit to the mainUnit of currency transacted
     * hence, we divide by 100
     */
    public static function formatAmountDown(float|string $amount): float
    {
        return (float) $amount / 100;
    }
}